@@ -22,11 +22,6 @@
 
 .. rubric:: Contents
 
-* :class:`_AdbTransactionInfo`
-* :class:`_FileSyncTransactionInfo`
-
-    * :meth:`_FileSyncTransactionInfo.can_add_to_send_buffer`
-
 * :class:`AdbDevice`
 
     * :meth:`AdbDevice._close`
@@ -54,17 +49,17 @@
     * :meth:`AdbDevice.list`
     * :meth:`AdbDevice.pull`
     * :meth:`AdbDevice.push`
+    * :meth:`AdbDevice.root`
     * :meth:`AdbDevice.shell`
     * :meth:`AdbDevice.stat`
     * :meth:`AdbDevice.streaming_shell`
 
 * :class:`AdbDeviceTcp`
+* :class:`AdbDeviceUsb`
 
 """
 
 
-from collections import namedtuple
-from contextlib import contextmanager
 import io
 import logging
 import os
@@ -77,130 +72,16 @@ from . import exceptions
 from .adb_message import AdbMessage, checksum, unpack
 from .handle.base_handle import BaseHandle
 from .handle.tcp_handle import TcpHandle
-
+from .hidden_helpers import FILE_TYPES, DeviceFile, _AdbTransactionInfo, _FileSyncTransactionInfo, _open
 
 try:
-    FILE_TYPES = (file, io.IOBase)
-except NameError:  # pragma: no cover
-    FILE_TYPES = (io.IOBase,)
+    from .handle.usb_handle import UsbHandle
+except ImportError:
+    UsbHandle = None
+
 
 _LOGGER = logging.getLogger(__name__)
 
-DeviceFile = namedtuple('DeviceFile', ['filename', 'mode', 'size', 'mtime'])
-
-
-@contextmanager
-def _open(name, mode='r'):
-    """Handle opening and closing of files and IO streams.
-
-    Parameters
-    ----------
-    name : str, io.IOBase
-        The name of the file *or* an IO stream
-    mode : str
-        The mode for opening the file
-
-    Yields
-    ------
-    io.IOBase
-        The opened file *or* the IO stream
-
-    """
-    try:
-        opened = open(name, mode) if isinstance(name, str) else None
-        if isinstance(name, str):
-            yield opened
-        else:
-            yield name
-    finally:
-        if isinstance(name, str):
-            opened.close()
-        else:
-            name.close()
-
-
-class _AdbTransactionInfo(object):  # pylint: disable=too-few-public-methods
-    """A class for storing info and settings used during a single ADB "transaction."
-
-    Parameters
-    ----------
-    local_id : int
-        The ID for the sender (i.e., the device running this code)
-    remote_id : int
-        The ID for the recipient
-    timeout_s : float, None
-        Timeout in seconds for sending and receiving packets, or ``None``; see :meth:`BaseHandle.bulk_read() <adb_shell.handle.base_handle.BaseHandle.bulk_read>`
-        and :meth:`BaseHandle.bulk_write() <adb_shell.handle.base_handle.BaseHandle.bulk_write>`
-    total_timeout_s : float
-        The total time in seconds to wait for a command in ``expected_cmds`` in :meth:`AdbDevice._read`
-
-    Attributes
-    ----------
-    local_id : int
-        The ID for the sender (i.e., the device running this code)
-    remote_id : int
-        The ID for the recipient
-    timeout_s : float, None
-        Timeout in seconds for sending and receiving packets, or ``None``; see :meth:`BaseHandle.bulk_read() <adb_shell.handle.base_handle.BaseHandle.bulk_read>`
-        and :meth:`BaseHandle.bulk_write() <adb_shell.handle.base_handle.BaseHandle.bulk_write>`
-    total_timeout_s : float
-        The total time in seconds to wait for a command in ``expected_cmds`` in :meth:`AdbDevice._read`
-
-    """
-    def __init__(self, local_id, remote_id, timeout_s=None, total_timeout_s=constants.DEFAULT_TOTAL_TIMEOUT_S):
-        self.local_id = local_id
-        self.remote_id = remote_id
-        self.timeout_s = timeout_s
-        self.total_timeout_s = total_timeout_s
-
-
-class _FileSyncTransactionInfo(object):  # pylint: disable=too-few-public-methods
-    """A class for storing info used during a single FileSync "transaction."
-
-    Parameters
-    ----------
-    recv_message_format : bytes
-        The FileSync message format
-
-    Attributes
-    ----------
-    recv_buffer : bytearray
-        A buffer for storing received data
-    recv_message_format : bytes
-        The FileSync message format
-    recv_message_size : int
-        The FileSync message size
-    send_buffer : bytearray
-        A buffer for storing data to be sent
-    send_idx : int
-        The index in ``recv_buffer`` that will be the start of the next data packet sent
-
-    """
-    def __init__(self, recv_message_format):
-        self.send_buffer = bytearray(constants.MAX_ADB_DATA)
-        self.send_idx = 0
-
-        self.recv_buffer = bytearray()
-        self.recv_message_format = recv_message_format
-        self.recv_message_size = struct.calcsize(recv_message_format)
-
-    def can_add_to_send_buffer(self, data_len):
-        """Determine whether ``data_len`` bytes of data can be added to the send buffer without exceeding :const:`constants.MAX_ADB_DATA`.
-
-        Parameters
-        ----------
-        data_len : int
-            The length of the data to be potentially added to the send buffer (not including the length of its header)
-
-        Returns
-        -------
-        bool
-            Whether ``data_len`` bytes of data can be added to the send buffer without exceeding :const:`constants.MAX_ADB_DATA`
-
-        """
-        added_len = self.recv_message_size + data_len
-        return self.send_idx + added_len < constants.MAX_ADB_DATA
-
 
 class AdbDevice(object):
     """A class with methods for connecting to a device and executing ADB commands.
@@ -436,6 +317,25 @@ class AdbDevice(object):
             for line in stream:
                 yield line
 
+    def root(self, timeout_s=None, total_timeout_s=constants.DEFAULT_TOTAL_TIMEOUT_S):
+        """Gain root access.
+
+        The device must be rooted in order for this to work.
+
+        Parameters
+        ----------
+        timeout_s : float, None
+            Timeout in seconds for sending and receiving packets, or ``None``; see :meth:`BaseHandle.bulk_read() <adb_shell.handle.base_handle.BaseHandle.bulk_read>`
+            and :meth:`BaseHandle.bulk_write() <adb_shell.handle.base_handle.BaseHandle.bulk_write>`
+        total_timeout_s : float
+            The total time in seconds to wait for a ``b'CLSE'`` or ``b'OKAY'`` command in :meth:`AdbDevice._read`
+
+        """
+        if not self.available:
+            raise exceptions.AdbConnectionError("ADB command not sent because a connection to the device has not been established.  (Did you call `AdbDevice.connect()`?)")
+
+        self._service(b'root', b'', timeout_s, total_timeout_s, False)
+
     def shell(self, command, timeout_s=None, total_timeout_s=constants.DEFAULT_TOTAL_TIMEOUT_S, decode=True):
         """Send an ADB shell command to the device.
 
@@ -1267,3 +1167,42 @@ class AdbDeviceTcp(AdbDevice):
     def __init__(self, host, port=5555, default_timeout_s=None, banner=None):
         handle = TcpHandle(host, port, default_timeout_s)
         super(AdbDeviceTcp, self).__init__(handle, banner)
+
+
+class AdbDeviceUsb(AdbDevice):
+    """A class with methods for connecting to a device via USB and executing ADB commands.
+
+    Parameters
+    ----------
+    serial : str, None
+        The USB device serial ID
+    port_path : TODO, None
+        TODO
+    default_timeout_s : float, None
+        Default timeout in seconds for USB packets, or ``None``
+    banner : str, bytes, None
+        The hostname of the machine where the Python interpreter is currently running; if
+        it is not provided, it will be determined via ``socket.gethostname()``
+
+    Raises
+    ------
+    adb_shell.exceptions.InvalidHandleError
+        Raised if package was not installed with the "usb" extras option (``pip install adb-shell[usb]``)
+
+    Attributes
+    ----------
+    _available : bool
+        Whether an ADB connection to the device has been established
+    _banner : bytearray, bytes
+        The hostname of the machine where the Python interpreter is currently running
+    _handle : UsbHandle
+        The handle that is used to connect to the device
+
+    """
+
+    def __init__(self, serial=None, port_path=None, default_timeout_s=None, banner=None):
+        if UsbHandle is None:
+            raise exceptions.InvalidHandleError("To enable USB support you must install this package via `pip install adb-shell[usb]`")
+
+        handle = UsbHandle.find_adb(serial, port_path, default_timeout_s)
+        super(AdbDeviceUsb, self).__init__(handle, banner)
