@@ -91,3 +91,39 @@ class TcpTimeoutException(Exception):
     """TCP connection timed read/write operation exceeded the allowed time.
 
     """
+
+
+class UsbDeviceNotFoundError(Exception):
+    """TODO
+
+    """
+
+
+class UsbReadFailedError(Exception):
+    """TODO
+
+    Parameters
+    ----------
+    msg : str
+        The error message
+    usb_error : libusb1.USBError
+        An exception from ``libusb1``
+
+    Attributes
+    ----------
+    usb_error : libusb1.USBError
+        An exception from ``libusb1``
+
+    """
+    def __init__(self, msg, usb_error):
+        super(UsbReadFailedError, self).__init__(msg)
+        self.usb_error = usb_error
+
+    def __str__(self):
+        return '%s: %s' % (super(UsbReadFailedError, self).__str__(), str(self.usb_error))
+
+
+class UsbWriteFailedError(Exception):
+    """:meth:`adb_shell.handle.usb_handle.UsbHandle.bulk_write` failed.
+
+    """
